#!/usr/bin/perl -i.orig

use v5.10;
use strict;
use warnings;
use Getopt::Long;
use File::Find;

my $version = "1.2";

@ARGV < 3 and die "Usage: $0 -m <mapping> [...] path ...\nStopped";

my @mappings;
my $verbose;

GetOptions ("map=s" => \@mappings,
	    "verbose"  => \$verbose);

my (@Old, @New);

for (@mappings) {
    my ($old,$new) = split(/\|/);
    push @Old, $old;
    push @New, $new;
}

sub do_replace {
    return unless /\.(mpl|m3u|m3u8|xml)$/i;

    local @ARGV;
    push @ARGV, $_;		# $_ is filename

    my ($type) = ($_ =~ /\.(.+)$/);
    while (<>) {
	for my $i (0 .. $#Old) {

	    # do the mapping transformation
	    s/\Q$Old[$i]\E/$New[$i]/	if defined $New[$i];

	    # Target: Windoze
	    if (defined $New[$i] and ($New[$i] =~ /^[A-Za-z]:/ or $New[$i] =~ /\\/)) {
		if ($type eq 'mpl') {
		    if (my ($pre,$path,$post) = ($_ =~ m#^(<Field Name="Filename">)([^<]+)(</Field>.*)#ms)) {
			$path =~ s#/#\\#g;
			$path =~ s#(?<=.)\\+#\\#g;
			$_ = join '', $pre, $path, $post;
		    }
		}
		else {
		    s#/#\\#g;
		    s#(?<=.)\\+#\\#g;
		}
	    }

	    # Target: Mac
	    else {
		if ($type eq 'mpl') {
		    if (my ($pre,$path,$post) = ($_ =~ m#^(<Field Name="Filename">)([^<]+)(</Field>.*)#ms)) {
			$path =~ s#\\#/#g;
			$path =~ s#/+#/#g;
			$_ = join '', $pre, $path, $post;
		    }
		}
		elsif ($type eq 'xml') {
		    s!^(\s*<key>(?:Location|Music Folder)</key><string>)file://localhost!$1!;
		}
		else {
		    s#\\#/#g;
		    s#/+#/#g;
		}
	    }
	}
	print $_;
    }
    #close(OLD);
}

find(\&do_replace, @ARGV);
